#include "io.h"
#include "fileOperations.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char *readLine() {
    int size = 0;
    int allocated = 16;
    char *buffer = (char*)malloc( allocated );
    char c;
    while ( ( c = getc( stdin ) ) != '\n' ) {
        buffer[ size++ ] = c;
        if ( size == allocated ) {
            allocated += allocated;
            char *tmp = (char*)realloc( buffer, allocated );
            if ( !tmp ) {
                free( buffer );
                return NULL;
            }
            buffer = tmp;
        }
    }
    buffer[ size ] = '\0';
    return buffer;
}

void printInfo( Game *game ) {
    const char *name = game->gameName ? game->gameName : "--no-game--";
    printf( "Game: %s\tHealth: %d\t\tWeapon: %s (%d)\n",
            name,
            game->player.health,
            weaponType( game->player.weapon.type ),
            game->player.weapon.attack );
}

void printBorder() {
    for ( int i = 0; i < 79; ++i )
        putc( '=', stdout );
    putc( '\n', stdout );
}
void printLine() {
    for ( int i = 0; i < 79; ++i )
        putc( '-', stdout );
    putc( '\n', stdout );
}

void frameBegin( Game *game ) {
    // Use different clear command on different operating systems.
    // Lets assume that only Windows (cls) and *nix (clear)
    // OS can be used.
    system(
#if _WIN32
        "cls"
#else
        "clear"
#endif
        );

    printBorder();
    printInfo( game );
    printBorder();
    if ( game->notification ) {
        while ( game->notification ) {
            int length;
            length = printf( ">>>>\t%s", game->notification->msg, &length );
            for ( ; length < 80 - 8; ++length )
                putc( ' ', stdout );
            printf( "<<<<\n" );
            Notification *old = game->notification;
            game->notification = old->next;
            free( old );
        }
        printLine();
    }

}
void frameEnd() {
    printBorder();
}

int loadGamePrompt( Game *game ) {
    frameBegin( game );
    puts( "Write filename of a game map." );
    frameEnd();
    printf( ">> " );
    char *name = readLine();
    int r = 0;
    if ( name ) {
        r = loadGame( name, game );
    }
    if ( !r )
        addGameNotification( game, "Cannot read file." );
    else
        addGameNotification( game, "Game loaded." );
    free( name );

    return r;
}
int saveGamePrompt( Game *game ) {
    frameBegin( game );
    puts( "Write filename of a game map." );
    frameEnd();
    printf( ">> " );
    char *name = readLine();
    int r = 0;
    if ( name ) {
        r = saveGame( name, game );
    }
    if ( !r )
        addGameNotification( game, "Cannot write file." );
    else
        addGameNotification( game, "Game saved" );
    free( name );

    return r;
}

void help( Game *game ) {
    addGameNotification( game, "Press enter to quit" );
    frameBegin( game );
    puts( "Help" );
    puts( "\tOn the map, you can use keys W, S, A, and D to move." );
    puts( "\tWhen you press M, main menu is launched." );
    puts( "Map symbols" );
    puts( "\tPlayer is represented by character `i`." );
    puts( "\tWall is represented by character `#`." );
    puts( "\tWeapons are represented by characters:" );
    puts( "\t\ta - axe" );
    puts( "\t\tb - bow" );
    puts( "\t\ts - sword" );
    puts( "\tHealing point is represented by character `+`." );
    puts( "\tPowerpack is represented by character `@`." );
    frameEnd();
    getc( stdin );
}

char readCommand() {
    char c;
    printf( "> " );
    do {
        c = getc( stdin );
    } while ( '\n' == c );
    return c;
}

void removeTrailingNewline() {
    char c;
    do {
        c = getc( stdin );
    } while ( '\n' != c );
}

void printGame( Game *game ) {
    frameBegin( game );
    for ( int y = 0; y < game->height; ++y ) {
        for ( int x = 0; x < game->width; ++x ) {
            if ( x == game->player.position.x && y == game->player.position.y )
                putc( 'i', stdout );
            else
                putc( readMapAt( game, x, y ), stdout );
        }
        putc( '\n', stdout );
    }
    frameEnd();
}

void showMessage( Game *game, const char *msg ) {
    frameBegin( game );
    puts( msg );
    frameEnd();
}

void addGameNotification( Game *game, const char *notification ) {
    Notification *n = (Notification *)malloc( sizeof( Notification ) );
    n->msg = notification;
    n->next = NULL;
    Notification **last = &game->notification;
    while ( *last ) {
        last = &( *last )->next;
    }
    *last = n;
}

enum MenuEvents mainMenu( Game *game ) {
    frameBegin( game );

    puts( "\t(L)oad map." );
    puts( "\t(R)eload map." );
    puts( "\t(S)ave map." );
    puts( "\t(P)lay." );
    puts( "\t(H)elp." );
    puts( "\t(Q)uit." );

    frameEnd( game );
    char cmd = readCommand();
    removeTrailingNewline();
    switch ( cmd ) {
    case 'l':
    case 'L':
        return EvLoadFile;
    case 'r':
    case 'R':
        return EvReloadFile;
    case 's':
    case 'S':
        return EvSaveFile;
    case 'q':
    case 'Q':
        return EvQuit;
    case 'p':
    case 'P':
        return EvPlay;
    case 'h':
    case 'H':
        return EvHelp;
    default:
        addGameNotification( game, "Unknown option" );
        return EvUnknown;
    }
}
